@@ -5,17 +5,19 @@ from typing import List
 import boto3
 import humanize
 import durations
+from botocore.client import BaseClient
 
-from aws_hashicorp_packer_reaper.aws import EC2Instance
+from typing import Tuple
+from aws_hashicorp_packer_reaper.aws import EC2Instance, Tag, ReaperTagFilter
 from aws_hashicorp_packer_reaper.logger import log
+from aws_hashicorp_packer_reaper.schema import validate
 
 
-def list_packer_instances(ec2: object) -> List[EC2Instance]:
+def list_packer_instances(ec2: BaseClient, tags: Tuple[Tag]) -> List[EC2Instance]:
     paginator = ec2.get_paginator("describe_instances")
     instances = []
-    for response in paginator.paginate(
-        Filters=[{"Name": "tag:Name", "Values": ["Packer Builder"]}]
-    ):
+
+    for response in paginator.paginate(Filters=ReaperTagFilter(tags).to_api()):
         for reservation in response["Reservations"]:
             for instance in map(lambda i: EC2Instance(i), reservation["Instances"]):
                 log.debug(
@@ -38,10 +40,14 @@ def expired_packer_instances(
     )
 
 
-def stop_expired_instances(ec2: object, dry_run: bool, older_than: timedelta):
+def stop_expired_instances(
+    ec2: BaseClient, dry_run: bool, older_than: timedelta, tags: Tuple[Tag]
+):
     count = 0
     instances = expired_packer_instances(
-        list_packer_instances(ec2), ["running"], older_than
+        list_packer_instances(ec2, tags),
+        ["running"],
+        older_than,
     )
     for instance in instances:
         log.info(
@@ -55,9 +61,13 @@ def stop_expired_instances(ec2: object, dry_run: bool, older_than: timedelta):
     log.info(f"total of {len(instances)} running instances stopped")
 
 
-def terminate_expired_instances(ec2: object, dry_run: bool, older_than: timedelta):
+def terminate_expired_instances(
+    ec2: BaseClient, dry_run: bool, older_than: timedelta, tags: Tuple[Tag]
+):
     instances = expired_packer_instances(
-        list_packer_instances(ec2), ["running", "stopped"], older_than
+        list_packer_instances(ec2, tags),
+        ["running", "stopped"],
+        older_than,
     )
     for instance in instances:
         log.info(
@@ -75,15 +85,21 @@ operation = {"stop": stop_expired_instances, "terminate": terminate_expired_inst
 
 def handler(request, _):
     log.setLevel(os.getenv("LOG_LEVEL", "INFO"))
-    dry_run = request.get("dry_run", False)
-    older_than = durations.Duration(request.get("older_than", "2h"))
-    mode = request.get("mode", "stop")
 
-    operation[mode](
-        ec2=boto3.client("ec2"),
-        dry_run=dry_run,
-        older_than=timedelta(seconds=older_than.seconds),
-    )
+    if validate(request):
+        dry_run = request.get("dry_run", False)
+        older_than = durations.Duration(request.get("older_than"))
+        mode = request.get("mode", "stop")
+        tags = request.get("tags", [])
+        tags = tuple(Tag.from_string(s) for s in request.get("tags", []))
 
-if __name__ == '__main__':
-    handler({"mode":"stop", "older_than": "1m", "dry_run": "true"}, {})
+        operation[mode](
+            ec2=boto3.client("ec2"),
+            dry_run=dry_run,
+            older_than=timedelta(seconds=older_than.seconds),
+            tags=tags,
+        )
+
+
+if __name__ == "__main__":
+    handler({"mode": "stop", "older_than": "1m", "dry_run": True, "tags": []}, {})
